@@ -35,7 +35,8 @@ static int odmr_ok (int sock, char *argbuf)
 {
     int ok;
 
-    ok = SMTP_ok(sock, SMTP_MODE);
+    (void)argbuf;
+    ok = SMTP_ok(sock, SMTP_MODE, TIMEOUT_DEFAULT);
     if (ok == SM_UNRECOVERABLE)
 	return(PS_PROTOCOL);
     else
@@ -51,6 +52,7 @@ static int odmr_getrange(int sock, struct query *ctl, const char *id,
     char buf [MSGBUFSIZE+1];
     struct idlist *qnp;		/* pointer to Q names */
 
+    (void)id;
     if ((ok = SMTP_ehlo(sock, SMTP_MODE, fetchmailhost, 
 			ctl->server.esmtp_name, ctl->server.esmtp_password,
 			&opts)))
@@ -80,11 +82,11 @@ static int odmr_getrange(int sock, struct query *ctl, const char *id,
      */
     buf[0] = '\0';
     for (qnp = ctl->domainlist; qnp; qnp = qnp->next)
-	if (strlen(buf) + strlen((const char *)qnp->id) + 1 >= sizeof(buf))
+	if (strlen(buf) + strlen(qnp->id) + 1 >= sizeof(buf))
 	    break;
 	else
 	{
-	    strcat(buf, (const char *)qnp->id);
+	    strcat(buf, qnp->id);
 	    strcat(buf, ",");
 	}
     buf[strlen(buf) - 1] = '\0';	/* nuke final comma */
