@@ -46,11 +46,14 @@ enum {
     LA_SSLPROTO,
     LA_SSLCERTCK,
     LA_SSLCERTPATH,
+    LA_SSLCOMMONNAME,
     LA_SSLFINGERPRINT,
     LA_FETCHSIZELIMIT,
     LA_FASTUIDL,
     LA_LIMITFLUSH,
-    LA_IDLE
+    LA_IDLE,
+    LA_NOSOFTBOUNCE,
+    LA_SOFTBOUNCE
 };
 
 /* options still left: CgGhHjJoORTWxXYz */
@@ -77,6 +80,8 @@ static const struct option longoptions[] = {
   {"pidfile",	required_argument, (int *) 0, LA_PIDFILE },
   {"postmaster",required_argument, (int *) 0, LA_POSTMASTER },
   {"nobounce",	no_argument,	   (int *) 0, LA_NOBOUNCE },
+  {"nosoftbounce", no_argument,	   (int *) 0, LA_NOSOFTBOUNCE },
+  {"softbounce", no_argument,	   (int *) 0, LA_SOFTBOUNCE },
 
   {"protocol",	required_argument, (int *) 0, 'p' },
   {"proto",	required_argument, (int *) 0, 'p' },
@@ -125,6 +130,7 @@ static const struct option longoptions[] = {
   {"sslproto",	 required_argument, (int *) 0, LA_SSLPROTO },
   {"sslcertck", no_argument,	   (int *) 0, LA_SSLCERTCK },
   {"sslcertpath",   required_argument, (int *) 0, LA_SSLCERTPATH },
+  {"sslcommonname",    required_argument, (int *) 0, LA_SSLCOMMONNAME },
   {"sslfingerprint",   required_argument, (int *) 0, LA_SSLFINGERPRINT },
 #endif
 
@@ -298,6 +304,12 @@ int parsecmdline (int argc /** argument count */,
 	case LA_NOBOUNCE:
 	    run.bouncemail = FALSE;
 	    break;
+	case LA_NOSOFTBOUNCE:
+	    run.softbounce = FALSE;
+	    break;
+	case LA_SOFTBOUNCE:
+	    run.softbounce = TRUE;
+	    break;
 	case 'p':
 	    /* XXX -- should probably use a table lookup here */
 	    if (strcasecmp(optarg,"auto") == 0)
@@ -362,6 +374,8 @@ int parsecmdline (int argc /** argument count */,
 		ctl->server.authenticate = A_KERBEROS_V4;
 	    else if (strcmp(optarg, "ssh") == 0)
 		ctl->server.authenticate = A_SSH;
+	    else if (strcasecmp(optarg, "external") == 0)
+		ctl->server.authenticate = A_EXTERNAL;
 	    else if (strcmp(optarg, "otp") == 0)
 		ctl->server.authenticate = A_OTP;
 	    else if (strcmp(optarg, "opie") == 0)
@@ -537,6 +551,10 @@ int parsecmdline (int argc /** argument count */,
 	    ctl->sslcertpath = prependdir(optarg, currentwd);
 	    break;
 
+	case LA_SSLCOMMONNAME:
+	    ctl->sslcommonname = xstrdup(optarg);
+	    break;
+
 	case LA_SSLFINGERPRINT:
 	    ctl->sslfingerprint = xstrdup(optarg);
 	    break;
@@ -599,6 +617,8 @@ int parsecmdline (int argc /** argument count */,
 	P(GT_("      --pidfile     specify alternate PID (lock) file\n"));
 	P(GT_("      --postmaster  specify recipient of last resort\n"));
 	P(GT_("      --nobounce    redirect bounces from user to postmaster.\n"));
+	P(GT_("      --nosoftbounce fetchmail deletes permanently undeliverable messages.\n"));
+	P(GT_("      --softbounce  keep permanently undeliverable messages on server (default).\n"));
 #ifdef CAN_MONITOR
 	P(GT_("  -I, --interface   interface required specification\n"));
 	P(GT_("  -M, --monitor     monitor interface for activity\n"));
@@ -609,8 +629,9 @@ int parsecmdline (int argc /** argument count */,
 	P(GT_("      --sslcert     ssl client certificate\n"));
 	P(GT_("      --sslcertck   do strict server certificate check (recommended)\n"));
 	P(GT_("      --sslcertpath path to ssl certificates\n"));
+	P(GT_("      --sslcommonname  expect this CommonName from server (discouraged)\n"));
 	P(GT_("      --sslfingerprint fingerprint that must match that of the server's cert.\n"));
-	P(GT_("      --sslproto    force ssl protocol (ssl2/ssl3/tls1)\n"));
+	P(GT_("      --sslproto    force ssl protocol (SSL2/SSL3/TLS1)\n"));
 #endif
 	P(GT_("      --plugin      specify external command to open connection\n"));
 	P(GT_("      --plugout     specify external command to open smtp connection\n"));
