@@ -8,14 +8,15 @@
  * For license terms, see the file COPYING in this directory.
  */
 
+#include "config.h"
+#include "fetchmail.h"
+
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
 #include <signal.h>
-#include "fetchmail.h"
 #include "socket.h"
 #include "smtp.h"
-#include "config.h"
 #include "i18n.h"
 
 struct opt
@@ -38,34 +39,27 @@ static struct opt extensions[] =
 
 char smtp_response[MSGBUFSIZE];
 
-static char smtp_mode = 'S';
-
-void SMTP_setmode(char sl)
-/* set whether we are speaking SMTP or LMTP */
-{
-    smtp_mode = sl;
-}
-
-int SMTP_helo(int sock,const char *host)
+/* XXX: this must not be used for LMTP! */
+int SMTP_helo(int sock, char smtp_mode, const char *host)
 /* send a "HELO" message to the SMTP listener */
 {
   int ok;
 
   SockPrintf(sock,"HELO %s\r\n", host);
   if (outlevel >= O_MONITOR)
-      report(stdout, "SMTP> HELO %s\n", host);
-  ok = SMTP_ok(sock);
+      report(stdout, "%cMTP> HELO %s\n", smtp_mode, host);
+  ok = SMTP_ok(sock, smtp_mode, TIMEOUT_HELO);
   return ok;
 }
 
-static void SMTP_auth_error(int sock, char *msg)
+static void SMTP_auth_error(int sock, const char *msg)
 {
     SockPrintf(sock, "*\r\n");
     SockRead(sock, smtp_response, sizeof(smtp_response) - 1);
-    if (outlevel >= O_MONITOR) report(stdout, msg);
+    if (outlevel >= O_MONITOR) report(stdout, "%s", msg);
 }
 
-static void SMTP_auth(int sock, char *username, char *password, char *buf)
+static void SMTP_auth(int sock, char smtp_mode, char *username, char *password, char *buf)
 /* ESMTP Authentication support for fetchmail by Wojciech Polak */
 {	
 	int c;
@@ -105,11 +99,7 @@ static void SMTP_auth(int sock, char *username, char *password, char *buf)
 			report(stdout, GT_("Challenge decoded: %s\n"), b64buf);
 		hmac_md5(password, strlen(password),
 			 b64buf, strlen(b64buf), digest, sizeof(digest));
-#ifdef HAVE_SNPRINTF
 		snprintf(tmp, sizeof(tmp),
-#else
-		sprintf(tmp,
-#endif /* HAVE_SNPRINTF */
 		"%s %02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x",
 		username,  digest[0], digest[1], digest[2], digest[3],
 		digest[4], digest[5], digest[6], digest[7], digest[8],
@@ -118,18 +108,13 @@ static void SMTP_auth(int sock, char *username, char *password, char *buf)
 
 		to64frombits(b64buf, tmp, strlen(tmp));
 		SockPrintf(sock, "%s\r\n", b64buf);
-		SMTP_ok(sock);
+		SMTP_ok(sock, smtp_mode, TIMEOUT_DEFAULT);
 	}
 	else if (strstr(buf, "PLAIN")) {
 		int len;
 		if (outlevel >= O_MONITOR)
 			report(stdout, GT_("ESMTP PLAIN Authentication...\n"));
-#ifdef HAVE_SNPRINTF
-		snprintf(tmp, sizeof(tmp),
-#else
-		sprintf(tmp,
-#endif /* HAVE_SNPRINTF */
-		"^%s^%s", username, password);
+		snprintf(tmp, sizeof(tmp), "^%s^%s", username, password);
 
 		len = strlen(tmp);
 		for (c = len - 1; c >= 0; c--)
@@ -139,7 +124,7 @@ static void SMTP_auth(int sock, char *username, char *password, char *buf)
 		}
 		to64frombits(b64buf, tmp, len);
 		SockPrintf(sock, "AUTH PLAIN %s\r\n", b64buf);
-		SMTP_ok(sock);
+		SMTP_ok(sock, smtp_mode, TIMEOUT_DEFAULT);
 	}
 	else if (strstr(buf, "LOGIN")) {
 		if (outlevel >= O_MONITOR)
@@ -178,27 +163,35 @@ static void SMTP_auth(int sock, char *username, char *password, char *buf)
 		}
 		to64frombits(b64buf, password, strlen(password));
 		SockPrintf(sock, "%s\r\n", b64buf);
-		SMTP_ok(sock);
+		SMTP_ok(sock, smtp_mode, TIMEOUT_DEFAULT);
 	}
 	return;
 }
 
-int SMTP_ehlo(int sock, const char *host, char *name, char *password, int *opt)
+int SMTP_ehlo(int sock, char smtp_mode, const char *host, char *name, char *password, int *opt)
 /* send a "EHLO" message to the SMTP listener, return extension status bits */
 {
   struct opt *hp;
   char auth_response[511];
+  SIGHANDLERTYPE alrmsave;
+  const int tmout = (mytimeout >= TIMEOUT_HELO ? mytimeout : TIMEOUT_HELO);
 
   SockPrintf(sock,"%cHLO %s\r\n", (smtp_mode == 'S') ? 'E' : smtp_mode, host);
   if (outlevel >= O_MONITOR)
       report(stdout, "%cMTP> %cHLO %s\n", 
 	    smtp_mode, (smtp_mode == 'S') ? 'E' : smtp_mode, host);
-  
+
+  alrmsave = set_signal_handler(SIGALRM, null_signal_handler);
+  set_timeout(tmout);
+
   *opt = 0;
   while ((SockRead(sock, smtp_response, sizeof(smtp_response)-1)) != -1)
   {
       int  n = strlen(smtp_response);
 
+      set_timeout(0);
+      (void)set_signal_handler(SIGALRM, alrmsave);
+
       if (smtp_response[strlen(smtp_response)-1] == '\n')
 	  smtp_response[strlen(smtp_response)-1] = '\0';
       if (smtp_response[strlen(smtp_response)-1] == '\r')
@@ -207,59 +200,48 @@ int SMTP_ehlo(int sock, const char *host, char *name, char *password, int *opt)
 	  return SM_ERROR;
       smtp_response[n] = '\0';
       if (outlevel >= O_MONITOR)
-	  report(stdout, "SMTP< %s\n", smtp_response);
+	  report(stdout, "%cMTP< %s\n", smtp_mode, smtp_response);
       for (hp = extensions; hp->name; hp++)
 	  if (!strncasecmp(hp->name, smtp_response+4, strlen(hp->name))) {
 	      *opt |= hp->value;
 	      if (strncmp(hp->name, "AUTH ", 5) == 0)
-	      	strncpy(auth_response, smtp_response, sizeof(auth_response));
+		strncpy(auth_response, smtp_response, sizeof(auth_response));
 		auth_response[sizeof(auth_response)-1] = '\0';
 	  }
       if ((smtp_response[0] == '1' || smtp_response[0] == '2' || smtp_response[0] == '3') && smtp_response[3] == ' ') {
 	  if (*opt & ESMTP_AUTH)
-		SMTP_auth(sock, name, password, auth_response);
+		SMTP_auth(sock, smtp_mode, name, password, auth_response);
 	  return SM_OK;
       }
       else if (smtp_response[3] != '-')
 	  return SM_ERROR;
+
+      alrmsave = set_signal_handler(SIGALRM, null_signal_handler);
+      set_timeout(tmout);
   }
   return SM_UNRECOVERABLE;
 }
 
-int SMTP_from(int sock, const char *from, const char *opts)
+int SMTP_from(int sock, char smtp_mode, const char *from, const char *opts)
 /* send a "MAIL FROM:" message to the SMTP listener */
 {
     int ok;
     char buf[MSGBUFSIZE];
 
     if (from[0]=='<')
-#ifdef HAVE_SNPRINTF
-	snprintf(buf, sizeof(buf),
-#else
-	sprintf(buf,
-#endif /* HAVE_SNPRINTF */
-		"MAIL FROM:%s", from);
+	snprintf(buf, sizeof(buf), "MAIL FROM:%s", from);
     else
-#ifdef HAVE_SNPRINTF
-    snprintf(buf, sizeof(buf),
-#else
-    sprintf(buf,
-#endif /* HAVE_SNPRINTF */
-	    "MAIL FROM:<%s>", from);
+	snprintf(buf, sizeof(buf), "MAIL FROM:<%s>", from);
     if (opts)
-#ifdef HAVE_SNPRINTF
 	snprintf(buf+strlen(buf), sizeof(buf)-strlen(buf), "%s", opts);
-#else
-	strcat(buf, opts);
-#endif /* HAVE_SNPRINTF */
     SockPrintf(sock,"%s\r\n", buf);
     if (outlevel >= O_MONITOR)
 	report(stdout, "%cMTP> %s\n", smtp_mode, buf);
-    ok = SMTP_ok(sock);
+    ok = SMTP_ok(sock, smtp_mode, TIMEOUT_MAIL);
     return ok;
 }
 
-int SMTP_rcpt(int sock, const char *to)
+int SMTP_rcpt(int sock, char smtp_mode, const char *to)
 /* send a "RCPT TO:" message to the SMTP listener */
 {
   int ok;
@@ -267,11 +249,11 @@ int SMTP_rcpt(int sock, const char *to)
   SockPrintf(sock,"RCPT TO:<%s>\r\n", to);
   if (outlevel >= O_MONITOR)
       report(stdout, "%cMTP> RCPT TO:<%s>\n", smtp_mode, to);
-  ok = SMTP_ok(sock);
+  ok = SMTP_ok(sock, smtp_mode, TIMEOUT_RCPT);
   return ok;
 }
 
-int SMTP_data(int sock)
+int SMTP_data(int sock, char smtp_mode)
 /* send a "DATA" message to the SMTP listener */
 {
   int ok;
@@ -279,11 +261,11 @@ int SMTP_data(int sock)
   SockPrintf(sock,"DATA\r\n");
   if (outlevel >= O_MONITOR)
       report(stdout, "%cMTP> DATA\n", smtp_mode);
-  ok = SMTP_ok(sock);
+  ok = SMTP_ok(sock, smtp_mode, TIMEOUT_DATA);
   return ok;
 }
 
-int SMTP_rset(int sock)
+int SMTP_rset(int sock, char smtp_mode)
 /* send a "RSET" message to the SMTP listener */
 {
   int ok;
@@ -291,11 +273,11 @@ int SMTP_rset(int sock)
   SockPrintf(sock,"RSET\r\n");
   if (outlevel >= O_MONITOR)
       report(stdout, "%cMTP> RSET\n", smtp_mode);
-  ok = SMTP_ok(sock);
+  ok = SMTP_ok(sock, smtp_mode, TIMEOUT_DEFAULT);
   return ok;
 }
 
-int SMTP_quit(int sock)
+int SMTP_quit(int sock, char smtp_mode)
 /* send a "QUIT" message to the SMTP listener */
 {
   int ok;
@@ -303,15 +285,13 @@ int SMTP_quit(int sock)
   SockPrintf(sock,"QUIT\r\n");
   if (outlevel >= O_MONITOR)
       report(stdout, "%cMTP> QUIT\n", smtp_mode);
-  ok = SMTP_ok(sock);
+  ok = SMTP_ok(sock, smtp_mode, TIMEOUT_DEFAULT);
   return ok;
 }
 
-int SMTP_eom(int sock)
+int SMTP_eom(int sock, char smtp_mode)
 /* send a message data terminator to the SMTP listener */
 {
-  int ok;
-
   SockPrintf(sock,".\r\n");
   if (outlevel >= O_MONITOR)
       report(stdout, "%cMTP>. (EOM)\n", smtp_mode);
@@ -320,23 +300,21 @@ int SMTP_eom(int sock)
    * When doing LMTP, must process many of these at the outer level. 
    */
   if (smtp_mode == 'S')
-      ok = SMTP_ok(sock);
+      return SMTP_ok(sock, smtp_mode, TIMEOUT_EOM);
   else
-      ok = SM_OK;
-
-  return ok;
+      return SM_OK;
 }
 
 time_t last_smtp_ok = 0;
 
-int SMTP_ok(int sock)
+int SMTP_ok(int sock, char smtp_mode, int mintimeout)
 /* returns status of SMTP connection */
 {
     SIGHANDLERTYPE alrmsave;
 
     /* set an alarm for smtp ok */
     alrmsave = set_signal_handler(SIGALRM, null_signal_handler);
-    set_timeout(mytimeout);
+    set_timeout(mytimeout >= mintimeout ? mytimeout : mintimeout);
 
     while ((SockRead(sock, smtp_response, sizeof(smtp_response)-1)) != -1)
     {
