@@ -3,8 +3,6 @@
  *
  * Copyright 2001 by Eric S. Raymond
  * For license terms, see the file COPYING in this directory.
- *
- * 
  */
 
 #include  "config.h"
@@ -55,18 +53,33 @@ int mytimeout;		/* value of nonreponse timeout */
 struct msgblk msgblk;
 static int accept_count, reject_count;
 
+/** add given address to xmit_names if it exactly matches a full address
+ * \returns nonzero if matched */
+static int map_address(const char *addr, struct query *ctl, struct idlist **xmit_names)
+{
+    const char	*lname;
+
+    lname = idpair_find(&ctl->localnames, addr);
+    if (lname) {
+	if (outlevel >= O_DEBUG)
+	    report(stdout, GT_("mapped address %s to local %s\n"), addr, lname);
+	save_str(xmit_names, lname, XMIT_ACCEPT);
+	accept_count++;
+    }
+    return lname != NULL;
+}
+
+/** add given name to xmit_names if it matches declared localnames */
 static void map_name(const char *name, struct query *ctl, struct idlist **xmit_names)
-/* add given name to xmit_names if it matches declared localnames */
 /*   name:	 name to map */
 /*   ctl:	 list of permissible aliases */
 /*   xmit_names: list of recipient names parsed out */
 {
     const char	*lname;
-    int off = 0;
-    
-    lname = idpair_find(&ctl->localnames, name+off);
+
+    lname = idpair_find(&ctl->localnames, name);
     if (!lname && ctl->wildcard)
-	lname = name+off;
+	lname = name;
 
     if (lname != (char *)NULL)
     {
@@ -119,6 +132,11 @@ static void find_server_names(const char *hdr,
 	    if ((atsign = strchr((char *)cp, '@'))) {
 		struct idlist	*idp;
 
+		/* try to match full address first, this takes
+		 * precedence over localdomains and alias mappings */
+		if (map_address(cp, ctl, xmit_names))
+		    goto nomap;
+
 		/*
 		 * Does a trailing segment of the hostname match something
 		 * on the localdomains list?  If so, save the whole name
@@ -150,7 +168,7 @@ static void find_server_names(const char *hdr,
 		     * not, skip this name.  If it is, we'll keep
 		     * going and try to find a mapping to a client name.
 		     */
-		    if (!is_host_alias(atsign+1, ctl))
+		    if (!is_host_alias(atsign+1, ctl, &ai0))
 		    {
 			save_str(xmit_names, cp, XMIT_REJECT);
 			reject_count++;
@@ -190,6 +208,7 @@ static char *parse_received(struct query *ctl, char *bufp)
 {
     char *base, *ok = (char *)NULL;
     static char rbuf[HOSTLEN + USERNAMELEN + 4]; 
+    struct addrinfo *ai0;
 
 #define RBUF_WRITE(value) if (tp < rbuf+sizeof(rbuf)-1) *tp++=value
 
@@ -237,7 +256,7 @@ static char *parse_received(struct query *ctl, char *bufp)
 	 * recipient name after a following "for".  Otherwise
 	 * punt.
 	 */
-	if (is_host_alias(rbuf, ctl))
+	if (is_host_alias(rbuf, ctl, &ai0))
 	{
 	    if (outlevel >= O_DEBUG)
 		report(stdout, 
@@ -340,6 +359,23 @@ static char *parse_received(struct query *ctl, char *bufp)
 /* shared by readheaders and readbody */
 static int sizeticker;
 
+/** Print ticker based on a amount of data transferred of \a bytes.
+ * Increments \a *tickervar by \a bytes, and if it exceeds
+ * \a SIZETICKER, print a dot and reduce *tickervar by \a SIZETICKER. */
+static void print_ticker(int *tickervar, int bytes)
+{
+    *tickervar += bytes;
+    while (*tickervar >= SIZETICKER)
+    {
+	if (want_progress())
+	{
+	    fputc('.', stdout);
+	    fflush(stdout);
+	}
+	*tickervar -= SIZETICKER;
+    }
+}
+
 #define EMPTYLINE(s)   (((s)[0] == '\r' && (s)[1] == '\n' && (s)[2] == '\0') \
                        || ((s)[0] == '\n' && (s)[1] == '\0'))
 
@@ -425,7 +461,7 @@ int readheaders(int sock,
     {
 	char *line, *rline;
 
-	line = xmalloc(sizeof(buf));
+	line = (char *)xmalloc(sizeof(buf));
 	linelen = 0;
 	line[0] = '\0';
 	do {
@@ -512,6 +548,7 @@ int readheaders(int sock,
 	    /* check for end of headers */
 	    if (end_of_header(line))
 	    {
+eoh:
 		if (linelen != strlen (line))
 		    has_nuls = TRUE;
 		free(line);
@@ -525,15 +562,9 @@ int readheaders(int sock,
 	     */
 	    if (protocol->delimited && line[0] == '.' && EMPTYLINE(line+1))
 	    {
-		if (outlevel > O_SILENT)
-		    report(stdout,
-			   GT_("message delimiter found while scanning headers\n"));
 		if (suppress_readbody)
 		    *suppress_readbody = TRUE;
-		if (linelen != strlen (line))
-		    has_nuls = TRUE;
-		free(line);
-		goto process_headers;
+		goto eoh; /* above */
 	    }
 
 	    /*
@@ -574,32 +605,25 @@ int readheaders(int sock,
 	/* write the message size dots */
 	if ((outlevel > O_SILENT && outlevel < O_VERBOSE) && linelen > 0)
 	{
-	    sizeticker += linelen;
-	    while (sizeticker >= SIZETICKER)
-	    {
-		if (outlevel > O_SILENT && run.showdots && !run.use_syslog)
-		{
-		    fputc('.', stdout);
-		    fflush(stdout);
-		}
-		sizeticker -= SIZETICKER;
-	    }
+	    print_ticker(&sizeticker, linelen);
+	}
+
+	/*
+	 * Decode MIME encoded headers. We MUST do this before
+	 * looking at the Content-Type / Content-Transfer-Encoding
+	 * headers (RFC 2046).
+	 */
+	if ( ctl->mimedecode )
+	{
+	    char *tcp;
+	    UnMimeHeader(line);
+	    /* the line is now shorter. So we retrace back till we find
+	     * our terminating combination \n\0, we move backwards to
+	     * make sure that we don't catch some \n\0 stored in the
+	     * decoded part of the message */
+	    for (tcp = line + linelen - 1; tcp > line && (*tcp != 0 || tcp[-1] != '\n'); tcp--);
+	    if  (tcp > line) linelen = tcp - line;
 	}
-		/*
-		 * Decode MIME encoded headers. We MUST do this before
-		 * looking at the Content-Type / Content-Transfer-Encoding
-		 * headers (RFC 2046).
-		 */
-		if ( ctl->mimedecode )
-		{
-		    char *tcp;
-		    UnMimeHeader(line);
-		    /* the line is now shorter. So we retrace back till we find our terminating
-		     * combination \n\0, we move backwards to make sure that we don't catch som
-		     * \n\0 stored in the decoded part of the message */
-		    for(tcp = line + linelen - 1; tcp > line && (*tcp != 0 || tcp[-1] != '\n'); tcp--);
-		    if(tcp > line) linelen = tcp - line;
-		}
 
 
 	/* skip processing if we are going to retain or refuse this mail */
@@ -786,7 +810,7 @@ int readheaders(int sock,
 	if (!msgblk.headers)
 	{
 	    oldlen = linelen;
-	    msgblk.headers = xmalloc(oldlen + 1);
+	    msgblk.headers = (char *)xmalloc(oldlen + 1);
 	    (void) memcpy(msgblk.headers, line, linelen);
 	    msgblk.headers[oldlen] = '\0';
 	    free(line);
@@ -909,12 +933,12 @@ int readheaders(int sock,
 	}
     }
 
- process_headers:    
+process_headers:
 
-    if (retain_mail)
-    {
+    if (retain_mail) {
 	return(PS_RETAINED);
     }
+
     if (refuse_mail)
 	return(PS_REFUSED);
     /*
@@ -1176,8 +1200,8 @@ int readheaders(int sock,
 	if (n != -1)
 	{
 	    /*
-	     * This header is technically invalid under RFC822.
-	     * POP3, IMAP, etc. are not legal mail-parameter values.
+	     * We SHOULD (RFC-2821 sec. 4.4/p. 53) make sure to only use
+	     * IANA registered protocol names here.
 	     */
 	    snprintf(buf, sizeof(buf),
 		    "\tby %s with %s (fetchmail-%s",
@@ -1235,7 +1259,8 @@ int readheaders(int sock,
 	release_sink(ctl);
 	return(PS_IOERR);
     }
-    else if ((run.poll_interval == 0 || nodetach) && outlevel >= O_VERBOSE && !is_a_file(1) && !run.use_syslog)
+    
+    if (want_progress())
 	fputc('#', stdout);
 
     /* write error notifications */
@@ -1279,7 +1304,7 @@ int readheaders(int sock,
 		if (idp->val.status.mark == XMIT_RCPTBAD)
 		    errlen += strlen(idp->id) + 2;
 
-	    errmsg = xmalloc(errlen + 3);
+	    errmsg = (char *)xmalloc(errlen + 3);
 	    strcpy(errmsg, errhd);
 	    for (idp = msgblk.recipients; idp; idp = idp->next)
 		if (idp->val.status.mark == XMIT_RCPTBAD)
@@ -1305,9 +1330,12 @@ int readheaders(int sock,
     *cp++ = '\r';
     *cp++ = '\n';
     *cp++ = '\0';
-    stuffline(ctl, buf);
+    n = stuffline(ctl, buf);
 
-    return(PS_SUCCESS);
+    if ((size_t)n == strlen(buf))
+	return PS_SUCCESS;
+    else
+	return PS_SOCKET;
 }
 
 int readbody(int sock, struct query *ctl, flag forward, int len)
@@ -1335,6 +1363,9 @@ int readbody(int sock, struct query *ctl, flag forward, int len)
     while (protocol->delimited || len > 0)
     {
 	set_timeout(mytimeout);
+	/* XXX FIXME: for undelimited protocols that ship the size, such
+	 * as IMAP, we might want to use the count of remaining characters
+	 * instead of the buffer size -- not for fetchmail 6.3.X though */
 	if ((linelen = SockRead(sock, inbufp, sizeof(buf)-4-(inbufp-buf)))==-1)
 	{
 	    set_timeout(0);
@@ -1346,17 +1377,22 @@ int readbody(int sock, struct query *ctl, flag forward, int len)
 	/* write the message size dots */
 	if (linelen > 0)
 	{
-	    sizeticker += linelen;
-	    while (sizeticker >= SIZETICKER)
-	    {
-		if (outlevel > O_SILENT && run.showdots && !run.use_syslog)
-		{
-		    fputc('.', stdout);
-		    fflush(stdout);
-		}
-		sizeticker -= SIZETICKER;
-	    }
+	    print_ticker(&sizeticker, linelen);
 	}
+
+	/* Mike Jones, Manchester University, 2006:
+	 * "To fix IMAP MIME Messages in which fetchmail adds the remainder of
+	 * the IMAP packet including the ')' character (part of the IMAP)
+	 * Protocol causing the addition of an extra MIME boundary locally."
+	 *
+	 * However, we shouldn't do this for delimited protocols:
+	 * many POP3 servers (Microsoft, qmail) goof up message sizes
+	 * so we might end truncating messages prematurely.
+	 */
+	if (!protocol->delimited && linelen > len) {
+	    inbufp[len] = '\0';
+	}
+
 	len -= linelen;
 
 	/* check for end of message */
@@ -1401,11 +1437,11 @@ int readbody(int sock, struct query *ctl, flag forward, int len)
 
 	    if (n < 0)
 	    {
-		report(stdout, GT_("writing message text\n"));
+		report(stdout, GT_("error writing message text\n"));
 		release_sink(ctl);
 		return(PS_IOERR);
 	    }
-	    else if (outlevel >= O_VERBOSE && !is_a_file(1) && !run.use_syslog)
+	    else if (want_progress())
 	    {
 		fputc('*', stdout);
 		fflush(stdout);
