@@ -434,7 +434,7 @@ int MimeBodyType(char *hdrs, int WantDecode)
 	    if (p == NULL) p = NxtHdr + strlen(NxtHdr);
 
 	    xfree(CntType);
-	    CntType = xmalloc(p-NxtHdr+1);
+	    CntType = (char *)xmalloc(p-NxtHdr+1);
 	    strlcpy(CntType, NxtHdr, p-NxtHdr+1);
 	    HdrsFound++;
 	}
@@ -668,6 +668,10 @@ int main(int argc, char *argv[])
   char *buffer, *buf_p;
   int nl_count, i, bodytype;
 
+  /* quench warnings about unused arguments */
+  (void)argc;
+  (void)argv;
+
 #ifdef DEBUG
   pid_t pid;
   FILE *fd_orig, *fd_conv;
@@ -703,7 +707,7 @@ int main(int argc, char *argv[])
     buf_p++;
     if ((unsigned)(buf_p - buffer) == BufSize) {
        /* Buffer is full! Get more room. */
-       buffer = xrealloc(buffer, BufSize+BUFSIZE_INCREMENT);
+       buffer = (char *)xrealloc(buffer, BufSize+BUFSIZE_INCREMENT);
        buf_p = buffer + BufSize;
        BufSize += BUFSIZE_INCREMENT;
     }
